﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Diagnostics;
using System.Threading;
using System.IO;
using BlackBoxFile;
using System.Net;

namespace Dispatch
{
    /// <summary>
    /// 包含AGV发送的相关数据
    /// </summary>
    public class AgvDataEventArgs : EventArgs
    {
        /// <summary>
        /// AGV编号
        /// </summary>
        public int agvID;
        /// <summary>
        /// 结束点地标号
        /// </summary>
        public int mark;
        /// <summary>
        /// 当前路线
        /// </summary>
        public int route;
        /// <summary>
        /// 无线地址
        /// </summary>
        public int address;//10 33 30 30 30 50 30 34 00 01 02 01 71 03
        /// <summary>
        /// 通信方式 1：串口   2：网络
        /// </summary>
        public int type;
        /// <summary>
        /// 叫料工位
        /// </summary>
        public int callStation;
        /// <summary>
        /// 物料ID
        /// </summary>
        public int materialID;
        /// <summary>
        /// 物料数量
        /// </summary>
        public int maNum;
        /// <summary>
        /// PADID
        /// </summary>
        public int PADID;
        /// <summary>
        /// 叫料ID
        /// </summary>
        public int callID;
    }

    /// <summary>
    /// v4数据
    /// </summary>
    public class V4DataEventArgs
    {
        /// <summary>
        /// 盒子编号
        /// </summary>
        public int num;


        /// <summary>
        /// buffer4第零位 Y01
        /// </summary>
        public int Y01;

        /// <summary>
        /// buffer4第1位 Y02
        /// </summary>
        public int Y02;

        /// <summary>
        /// buffer4第2位 Y03
        /// </summary>
        public int Y03;

        /// <summary>
        /// buffer4第3位 Y04
        /// </summary>
        public int Y04;

        /// <summary>
        /// buffer4第4位 Y05
        /// </summary>
        public int Y05;


        /// <summary>
        /// buffer4第5位 Y06
        /// </summary>
        public int Y06;

        /// <summary>
        /// buffer4第6位 Y07
        /// </summary>
        public int Y07;

        /// <summary>
        /// buffer4第7位 Y08
        /// </summary>
        public int Y08;


        /// <summary>
        /// buffer3第零位 Y09
        /// </summary>
        public int Y09;

        /// <summary>
        /// buffer3第1位 Y10
        /// </summary>
        public int Y10;

        /// <summary>
        /// buffer3第2位 Y11
        /// </summary>
        public int Y11;

        /// <summary>
        /// buffer3第3位 Y12
        /// </summary>
        public int Y12;

        /// <summary>
        /// buffer4第3位 Y13
        /// </summary>
        public int Y13;


        /// <summary>
        /// buffer3第5位 Y14
        /// </summary>
        public int Y14;

        /// <summary>
        /// buffer3第6位 Y15
        /// </summary>
        public int Y15;

        /// <summary>
        /// buffer3第7位 Y16
        /// </summary>
        public int Y16;



        /// <summary>
        /// buffer6第零位 X01
        /// </summary>
        public int X01;

        /// <summary>
        /// buffer6第1位 X02
        /// </summary>
        public int X02;

        /// <summary>
        /// buffer6第2位 X03
        /// </summary>
        public int X03;

        /// <summary>
        /// buffer6第3位 X04
        /// </summary>
        public int X04;

        /// <summary>
        /// buffer6第4位 X05
        /// </summary>
        public int X05;


        /// <summary>
        /// buffer6第5位 X06
        /// </summary>
        public int X06;

        /// <summary>
        /// buffer6第6位 X07
        /// </summary>
        public int X07;

        /// <summary>
        /// buffer6第7位 X08
        /// </summary>
        public int X08;


        /// <summary>
        /// buffer5第零位 X09
        /// </summary>
        public int X09;

        /// <summary>
        /// buffer5第1位 X10
        /// </summary>
        public int X10;

        /// <summary>
        /// buffer5第2位 X11
        /// </summary>
        public int X11;

        /// <summary>
        /// buffer5第3位 X12
        /// </summary>
        public int X12;

        /// <summary>
        /// buffer5第3位 X13
        /// </summary>
        public int X13;


        /// <summary>
        /// buffer5第5位 X14
        /// </summary>
        public int X14;

        /// <summary>
        /// buffer5第6位 X15
        /// </summary>
        public int X15;

        /// <summary>
        /// buffer5第7位 X16
        /// </summary>
        public int X16;

    }

    /// <summary>
    /// AGV反馈回来数据
    /// </summary>
    public class AgvDateEventArgs1 : EventArgs
    {
        /// <summary>
        /// AGV编号
        /// </summary>
        public int AgvId;

        /// <summary>
        /// AGV当前的路线
        /// </summary>
        public int AGVRoute;

        /// <summary>
        /// AGV当前地标
        /// </summary>
        public int AgvMark;

        /// <summary>
        /// AGV当前的状态
        /// </summary>
        public string AgvStatus;

        /// <summary>
        /// AGV报警的状态
        /// </summary>
        public string AgvAlarmStatus;
        /// <summary>
        /// 速度
        /// </summary>
        public string SpeedGrade;
        /// <summary>
        /// 电压
        /// </summary>
        public decimal Voltage;
        /// <summary>
        /// 电流
        /// </summary>
        public decimal Current;
        /// <summary>
        /// 电量
        /// </summary>
        public decimal Battery_Power;
        /// <summary>
        /// 温度
        /// </summary>
        public decimal Temperatrue;
        /// <summary>
        /// 控制命令帧
        /// </summary>
        public decimal ControlCommand;

        /// <summary>
        /// 通讯类型
        /// </summary>
        public int Type;

        public bool DispatchInfo=false;

        public DateTime updateTime = DateTime.Now;

        public bool IsOnline = true;

    }

    public class DataEventArgs : EventArgs
    {
        public byte[] data;
        public int type;
    }

    /// <summary>
    /// AGV调度类，主要用于放行待命的AGV
    /// </summary>
    public class Dispatcher
    {
        private const int COM_FUNC = 1;
        private const int COM_ADDRESS = 3;
        private const int COM_ROUTE = 7;
        private const int COM_MARK = 8;
        private const int COM_AGV = 11;
        private const int COM_CHECK = 12;

        private const int NET_FUNC = 2;
        private const int NET_PADID = 3;
        private const int NET_MAID = 4;
        private const int NET_CALLID = 5;
        private const int NET_ROUTE = 7;
        private const int NET_MARK = 8;
        private const int NET_MANUM = 10;
        private const int NET_AGV = 11;
        private const int NET_CHECK = 12;
        
        private byte[] send_buffer = new byte[14] { 0x10, 0, 0, 0x30, 0, 0, 0, 0x30, 0, 0, 0, 0, 0, 0x03 };
        private byte[] send_buffer2 = new byte[10] {0x10,0x61,0,0,0,0,0,0,0x71,03 };
      
        internal static CBlackBoxFile gs_blackBoxFile;      // 黑匣子
        private static bool gs_receive0x32Flag = false;
        private SerialPortEx port;
        private UDPCommunication UdpConnect;
        /*2016-12-27 薛德龙修改 串口发送间隔时间100,net暂不变,单位毫秒，后面这个时间统一改，不再一一备注*/
        static readonly int sendSplitTime = 100;
        /*2016-12-27 薛德龙修改*/
        /// <summary>
        /// 有无日志记录功能，true ： 有 false ： 无
        /// </summary>
        public static bool writeLogFlag = false;
        public static string dispatchMode = "";
        /// <summary>
        /// 调度构造函数
        /// </summary>
        public Dispatcher()
        {
            CSNetComm.MatchedParity += new CSNetComm.MatchedParityHandler(MatchedParity);
            
            port = new SerialPortEx();
            port.MatchedParity += new SerialPortEx.MatchedParityHandler(MatchedParity);

            port.serialPort1.BaudRate = 9600;
           
            port.ReadingData += new EventHandler((o, e) =>
            {
                OnReadingData(e);
            });

            port.WritingData += new EventHandler((o, e) =>
            {
                OnWritingData(e);
            });
            gs_blackBoxFile = new CBlackBoxFile();
        }

        public Dispatcher(string ip,int cport)
        {
            UdpConnect = new UDPCommunication(ip, cport, 10);
            UdpConnect.MatchedParity += new UDPCommunication.MatchedParityHandler(MatchedParity);
            UdpConnect.ReadingData += new EventHandler((o, e) =>
            {
                OnReadingData(e);
            });
            UdpConnect.WritingData += new EventHandler((o, e) =>
            {
                OnWritingData(e);
            });
            gs_blackBoxFile = new CBlackBoxFile();
        }

        #region 后来增加事件，用于通知是否在操作串口
        /// <summary>
        /// 串口写
        /// </summary>
        public event EventHandler WritingData;
        /// <summary>
        /// 串口读
        /// </summary>
        public event EventHandler ReadingData;
        void OnReadingData(EventArgs e)
        {
            if (ReadingData != null)
            {
                ReadingData(this, e);
            }
        }
        void OnWritingData(EventArgs e)
        {
            if (WritingData != null)
            {
                WritingData(this, e);
            }
        }
        /// <summary>
        /// 串口是否打开
        /// </summary>
        /// <returns></returns>
        public bool ComIsOpen()
        {
            if(port!=null)
            {
                return port.IsOpen();
            }
            return false;
        }
        #endregion

        #region 叫料相关事件
        public delegate void V4DataHandler(object sender, V4DataEventArgs e);
        /// <summary>
        /// V4对接协议
        /// </summary>
        public event V4DataHandler V4Data;

        private void OnV4Data(V4DataEventArgs e)
        {
            if (V4Data != null)
            {
                V4Data(this, e);
            }
        }



        public delegate void WorkLineCalledHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 产线叫料
        /// </summary>
        public event WorkLineCalledHandler WorkLineCalled;
        private void OnWorkLineCalled(AgvDataEventArgs e)
        {
            if (WorkLineCalled != null)
            {
                WorkLineCalled(this, e);
            }
        }

        public delegate void WorklineCancledHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 叫料点取消叫料
        /// </summary>
        public event WorklineCancledHandler WorkLineCancled;
        private void OnWorkLineCancled(AgvDataEventArgs e)
        {
            if (WorkLineCancled != null)
            {
                WorkLineCancled(this, e);
            }
        }

        public delegate void WorkLineReplyedHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 叫料点回应调度的放车请求
        /// </summary>
        public event WorkLineReplyedHandler WorkLineReplyed;
        private void OnWorkLineReplyed(AgvDataEventArgs e)
        {
            if (WorkLineReplyed != null)
            {
                WorkLineReplyed(this, e);
            }
        }
        public delegate void AskAgvWorkStatusHandler(object sender, AgvDateEventArgs1 e);
        /// <summary>
        /// 轮询AGV反馈回来的事件
        /// </summary>
        public event AskAgvWorkStatusHandler AskAgvWorkStatus;

        private void OnAskAgvWorkStatus(AgvDateEventArgs1 e)
        {
            if (AskAgvWorkStatus!=null)
             {
                 AskAgvWorkStatus(this,e);
             }
        }
        #endregion

        #region 调度相关事件
        /// <summary>
        /// 处理AgvReadEnding事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReadEndingHandler(object sender, AgvDataEventArgs e);

        /// <summary>
        /// 当结束点有AGV时发生
        /// </summary>
        public event AgvReadEndingHandler AgvReadEnding;
        private void OnAgvReadEnding(object sender, AgvDataEventArgs e)
        {
            if (AgvReadEnding != null)
            {
                AgvReadEnding(sender, e);
            }
        }

        /// <summary>
        /// 处理AgvReadEnding事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReadStopingHandler(object sender, AgvDataEventArgs e);

        /// <summary>
        /// 当停止点有AGV时发生
        /// </summary>
        public event AgvReadStopingHandler AgvReadStoping;
        private void OnAgvReadStoping(object sender, AgvDataEventArgs e)
        {
            if (AgvReadStoping != null)
            {
                AgvReadStoping(sender, e);
            }
        }

        /// <summary>
        /// 处理AgvReleased事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReleasedHandler(object sender, AgvDataEventArgs e);

        /// <summary>
        /// 当结束点AGV放行成功时发生
        /// </summary>
        public event AgvReleasedHandler AgvReleased;
        private void OnAgvReleased(object sender, AgvDataEventArgs e)
        {
            if (AgvReleased != null)
            {
                AgvReleased(sender, e);
            }
        }
        /// <summary>
        /// 处理AgvReleased事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReleasedHandler37(object sender, AgvDataEventArgs e);
        public event AgvReleasedHandler37 AgvReleased37;
        private void OnAgvReleased37(object sender, AgvDataEventArgs e)
        {
            if (AgvReleased37 != null)
            {
                AgvReleased37(sender, e);
            }
        }

        /// <summary>
        /// 处理AgvReleaseOnStop事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReleaseOnStopHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 当AGV停止点放行成功时发生
        /// </summary>
        public event AgvReleaseOnStopHandler AgvReleasedOnStop;
        private void OnAgvReleasedOnStop(object sender, AgvDataEventArgs e)
        {
            if (AgvReleasedOnStop != null)
            {
                AgvReleasedOnStop(sender, e);
            }
        }
        #endregion

        #region 调度相关函数

        /// <summary>
        /// /// <summary>
        /// 发送控制80的V4盒子的数据
        /// </summary>
        /// </summary>
        /// <param name="bit">从零开始第几位，0-7位为buffer[4]的8位，8-15位为buffer[3]的8位</param>
        /// <param name="value">0或者1</param>
        /// <param name="num">设备编号</param>
        public void SendV4_80(int bit,int value,int num)
        {
            byte[] buffer = new byte[14] { 0x10, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0, 0, 0, 0, 0, 0x03 };

            buffer[2] = (byte)num;

            if (bit > 7)
            {
                int bit1 = bit - 8;
                if (value == 1)
                {
                    //x |= (1 << y)
                    buffer[4] = (byte)(0x00 | (1 << bit1));
                }
                else if (value == 0)
                {//x&=!(1<<y)
                    buffer[4] = (byte)(0x00 & ((1 << bit1)));
                }

            }
            else 
            {
                if (value == 1)
                {
                    //x |= (1 << y)
                    buffer[3] = (byte)(0x00 | (1 << bit));
                }
                else if (value == 0)
                {//x&=!(1<<y)
                    buffer[3] = (byte)(0x00 & ((1 << bit)));
                }
            }

            if (UdpConnect != null)
            {
                UdpConnect.SendData(buffer, null);
            }
        }


        /// <summary>
        /// 用于发车成功后，通知叫料盒
        /// </summary>
        /// <param name="route"></param>
        /// <param name="address"></param>
        /// <param name="mark"></param>
        /// <param name="agv"></param>
        public void Send45ToWorkLine(int route, bool isAsk=false,int address = 0, int mark = 0, int agv = 0)
        {
            byte[] buffer = new byte[14] { 0x10, 0, 0x30, 0x30, 0x30, 0x50, 0x30, 0x30, 0, 0, 0, 0, 0, 0x03 };
            buffer[COM_FUNC] = 0x45;
            if (isAsk)
            {
                buffer[COM_FUNC] = 0x40;
               // route = 0;
            }
            buffer[COM_ADDRESS] = (byte)(address + 0x30);
            /*2016-12-27 薛德龙修改*/
            int Route = 0x3030 + route;
            buffer[COM_ROUTE - 1] = (byte)((Route >> 8) & 0xFF);
            buffer[COM_ROUTE] = (byte)(Route & 0xFF);
            /*2016-12-27 薛德龙修改*/
            buffer[COM_MARK] = (byte)((mark & 0xff00) >> 8);
            buffer[COM_MARK + 1] = (byte)(mark & 0x00ff);
            buffer[COM_AGV] = (byte)agv;
            buffer[COM_CHECK] = SerialPortEx.XOR_Check(buffer, 12);
            for (int i = 0; i < 1; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(sendSplitTime);
            }
        }
        /// <summary>
        /// 通知网络叫料端，已派车
        /// </summary>
        /// <param name="route">路线</param>
        /// <param name="terminalID">叫料终端</param>
        /// <param name="callID">叫料ID</param>
        /// <param name="agvID">AGVID</param>
        public void Send42ToWorkLineNet(int route, int terminalID, int callID, int agvID)
        {
            byte[] buffer = new byte[14] { 0x10, 0x0E, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            buffer[NET_FUNC] = 0x52;
            buffer[NET_PADID] = (byte)(terminalID);
            buffer[NET_CALLID] = (byte)(callID);
            buffer[NET_ROUTE] = (byte)(route);
            buffer[NET_AGV] = (byte)agvID;
            buffer[12] = SerialPortEx.XOR_Check(buffer, 12);

            for (int i = 0; i < 3; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(200);
            }
        }
        /// <summary>
        /// 通知网络叫料端，AGV已到目的地
        /// </summary>
        /// <param name="route">路线</param>
        /// <param name="terminalID">叫料终端</param>
        /// <param name="callID">叫料ID</param>
        /// <param name="agvID">AGVID</param>
        public void Send45ToWorkLineNet(int route, int terminalID, int callID, int agvID)
        {
            byte[] buffer = new byte[14] { 0x10, 0x0E, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            buffer[NET_FUNC] = 0x55;
            buffer[NET_PADID] = (byte)(terminalID);
            buffer[NET_CALLID] = (byte)(callID);
            buffer[NET_ROUTE] = (byte)(route);
            buffer[NET_AGV] = (byte)agvID;
            buffer[12] = SerialPortEx.XOR_Check(buffer, 12);

            for (int i = 0; i < 3; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(200);
            }
        }
        public static string ByteToHexString(byte[] buffer)
        {
            StringBuilder builder = new StringBuilder(buffer.Length * 3);
            foreach (byte data in buffer)
            {
                builder.Append(Convert.ToString(data, 16).PadLeft(2, '0').PadRight(3, ' '));
            }
            return builder.ToString().ToUpper();
        }
        /// <summary>
        /// 让放行灯亮
        /// </summary>
        /// <param name="route"></param>
        /// <param name="markID"></param>
        public void Send35ToWorkLineNet(int route, int markID)
        {
            byte[] buffer = new byte[14] { 0x10, 0x35, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            buffer[1] = 0x35;
            buffer[2] = 0x30;
            buffer[3] = 0x30;
            buffer[4] = 0x30;
            buffer[5] = 0x50;
            buffer[6] = 0x30;//
            buffer[7] = (byte)(route);//放行路线
            buffer[9] = (byte)markID;//放行地标
            buffer[12] = SerialPortEx.XOR_Check(buffer, 12);
            string sendMsg = ByteToHexString(buffer);
            for (int i = 0; i < 3; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(200);
            }
        }
        /// <summary>
        /// 熄灭放行按钮
        /// </summary>
        /// <param name="route"></param>
        /// <param name="markID"></param>
        public void Send38ToWorkLineNet(int route, int markID)
        {
            byte[] buffer = new byte[14] { 0x10, 0x35, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            buffer[1] = 0x38;
            buffer[2] = 0x30;
            buffer[3] = 0x30;
            buffer[4] = 0x30;
            buffer[5] = 0x50;
            buffer[6] = 0x30;//
            buffer[7] = (byte)(route);//放行路线
            buffer[9] = (byte)markID;//放行地标
            buffer[12] = SerialPortEx.XOR_Check(buffer, 12);
            string sendMsg = ByteToHexString(buffer);
            for (int i = 0; i < 3; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(200);
            }
        }
        /// <summary>
        /// 发送报警灯信号
        /// </summary>
        /// <param name="flag">1为响；0为不响</param>
        /// <param name="number">报警灯编号</param>
        /// <param name="ip">目标IP</param>
        public void AlarmStartOrStop(int flag, int number,string ip)
        {
            byte[] buffer = new byte[16] { 0x10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
            buffer[4] = (byte)(flag);
            buffer[14] = (byte)(number);
            buffer[15] = SerialPortEx.XOR_Check(buffer, 15);
            for (int i = 0; i < 5; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        IPEndPoint endpoint = new IPEndPoint(IPAddress.Parse(ip), 8899);
                        UdpConnect.SendData(buffer, endpoint);
                    }
                }
            }
            //        for (int i = 0; i < 10; i++)
            {
         //       port.Write(buffer, 0, buffer.Length);
                //            Thread.Sleep(50);
            }
            if (true)
            {
                #region 写日志
                //string str = "结束点放行AGV（ReleaseAgv）， 结束点地标： " + mark.ToString() + "路线： " + route.ToString() + "\n";
                string str = "";
                for (int i = 0; i < buffer.Length; i++)
                    str += Convert.ToString(buffer[i], 16).ToString() + " ";
                gs_blackBoxFile.WriteFile(10086, str);
                #endregion
            }
        }

        /// <summary>
        /// 开始串口监听，执行后才能进行数据收发
        /// </summary>
        /// <param name="portName">监听的串口号</param>
        public void StartListen2(string portName,int baudrate1)
        {
            if (port != null && !port.serialPort2.IsOpen)
            {
                if (!port.serialPort2.IsOpen)
                {
                    try
                    {
                        port.serialPort2.PortName = portName;
                        port.serialPort2.BaudRate = baudrate1;
                        port.Open1();
                    }
                    catch (System.Exception ex)
                    {
                        string logInfo = ex.Message;
                        StackTrace st = new StackTrace(true);
                        if (st.FrameCount > 1)
                        {
                            StackFrame sf = st.GetFrame(1);
                            logInfo += sf.GetFileName();
                            logInfo += sf.GetMethod().Name;
                            logInfo += sf.GetFileLineNumber();
                        }
                        gs_blackBoxFile.WriteFile(100, "7:" + logInfo);
                        throw new Exception(logInfo);
                    }
                }
            }
        }
        /// <summary>
        /// 开始串口监听，执行后才能进行数据收发
        /// </summary>
        /// <param name="portName">监听的串口号</param>
        public void StartListen(string portName)
        {
            if (port != null && !port.serialPort1.IsOpen)
            {
                if (!port.serialPort1.IsOpen)
                {
                    try
                    {
                        port.serialPort1.PortName = portName;
                        port.Open();
                    }
                    catch (System.Exception ex)
                    {
                        string logInfo = ex.Message;
                        StackTrace st = new StackTrace(true);
                        if (st.FrameCount > 1)
                        {
                            StackFrame sf = st.GetFrame(1);
                            logInfo += sf.GetFileName();
                            logInfo += sf.GetMethod().Name;
                            logInfo += sf.GetFileLineNumber();
                        }
                        gs_blackBoxFile.WriteFile(100, "7:" + logInfo);
                        throw new Exception(logInfo);
                    }
                }
            }
        }
        /// <summary>
        /// 开始网络监听
        /// </summary>
        /// <param name="ip">ip地址</param>
        /// <param name="port">端口</param>
        public void StartListenNet(string ip, int port)
        {
            CSNetComm.Start(ip, port);
        }
        /// <summary>
        /// 开始监听，执行后才能进行数据收发
        /// </summary>
        /// <param name="portName">监听的串口号</param>
        public void StartListenUDP()
        {
            UdpConnect.Open();
        }
        /// <summary>
        /// 关闭监听，关闭后将不能进行数据收发
        /// </summary>
        public void StopListen()
        {
            if (port != null && port.serialPort1.IsOpen)
            {
                try
                {
                    port.Close();
                }
                catch (System.Exception ex)
                {
                    string logInfo = ex.Message;
                    StackTrace st = new StackTrace(true);
                    if (st.FrameCount > 1)
                    {
                        StackFrame sf = st.GetFrame(1);
                        logInfo += sf.GetFileName();
                        logInfo += sf.GetMethod().Name;
                        logInfo += sf.GetFileLineNumber();
                    }
                    gs_blackBoxFile.WriteFile(100, "6:" + logInfo);
                    throw new Exception("关闭串口：" + logInfo);
                }

            }
        }
        /// <summary>
        /// 发送数据10 61询问
        /// </summary>
        public void SendAgvData61()
        {
            if(port!=null)
            {
                port.Write1(send_buffer2, 0, send_buffer2.Length); 
            }
        }
        /// <summary>
        /// 发送数据10 61询问
        /// </summary>
        public void SendAgvData61Net()
        {
           CSNetComm.SendBytes(0,send_buffer2, send_buffer2.Length);
        }
        /// <summary>
        /// 发送数据10 61询问
        /// </summary>
        public void SendAgvData61Udp()
        {
            if(UdpConnect!=null)
            {
                UdpConnect.SendData(send_buffer2,null);
            }
        }
        /// <summary>
        /// 待命点放行AGV函数，只有在结束点处有AGV时，放行AGV才能生效
        /// </summary>
        /// <param name="route">放行路线</param>
        /// <param name="address">AGV无线地址，默认0</param>
        /// <param name="mark">放行地标号</param>
        /// <param name="agv">放行AGV编号，默认0</param>
        /// <param name="low">低八位，默认0；注哪个工位要停，对应哪位为1</param>
        /// <param name="high">高八位，默认0</param>
        public void ReleaseAgv(int route, int mark, byte low = 0, byte high = 0, int address = 0, int agv = 0)
        {
            byte[] buffer = new byte[14] { 0x10, 0, 0x30, 0x30, 0x30, 0x50, 0x30, 0x30, 0, 0, 0, 0, 0, 0x03 };
            if (low != 0 || high != 0)
            {
                buffer[4] = high;
                buffer[5] = low;
            }
            buffer[COM_FUNC] = 0x31;
            buffer[COM_ADDRESS] = (byte)(address + 0x30);
            /*2016-12-27 薛德龙修改*/
            int Route = 0x3030 + route;
            buffer[COM_ROUTE - 1] = (byte)((Route >> 8) & 0xFF);
            buffer[COM_ROUTE] = (byte)(Route & 0xFF);
            /*2016-12-27 薛德龙修改*/
            buffer[COM_MARK] = (byte)((mark & 0xff00) >> 8);
            buffer[COM_MARK + 1] = (byte)(mark & 0x00ff);
            buffer[COM_AGV] = (byte)agv;
            buffer[COM_CHECK] = SerialPortEx.XOR_Check(buffer, 12);

            gs_receive0x32Flag = true;
            for (int i = 0; i < 5 && gs_receive0x32Flag; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(sendSplitTime);
             
            }
            if (writeLogFlag)
            {
                #region 写日志
                //string str = "结束点放行AGV（ReleaseAgv）， 结束点地标： " + mark.ToString() + "路线： " + route.ToString() + "\n";
                string str = "";
                for (int i = 0; i < buffer.Length; i++)
                    str += Convert.ToString(buffer[i], 16).ToString() + " ";
                gs_blackBoxFile.WriteFile(100, str);
                #endregion
            }
        }

        /// <summary>
        /// 停止点放行AGV函数
        /// </summary>
        /// <param name="mark">放行地标号</param>
        ///   <param name="mark">放行路线</param>
        /// <param name="address">AGV无线地址，默认0</param>
        /// <param name="agv">放行AGV编号，默认0</param>
        public void ReleaseAgvOnStop(int mark,int route, int address = 0, int agv = 0)
        {
            byte[] buffer = new byte[14] { 0x10, 0, 0x30, 0x30, 0x30, 0x50, 0x30, 0x30, 0, 0, 0, 0, 0, 0x03 };
            buffer[COM_FUNC] = 0x37;
            buffer[COM_ADDRESS] = (byte)(address + 0x30);
            /*2016-12-27 薛德龙修改*/
            int Route = 0x3030 + route;
            buffer[COM_ROUTE - 1] = (byte)((Route >> 8) & 0xFF);
            buffer[COM_ROUTE] = (byte)(Route & 0xFF);
            /*2016-12-27 薛德龙修改*/
            buffer[COM_MARK] = (byte)((mark & 0xff00) >> 8);
            buffer[COM_MARK + 1] = (byte)(mark & 0x00ff);
            buffer[COM_AGV] = (byte)agv;
            buffer[COM_CHECK] = SerialPortEx.XOR_Check(buffer, 12);
            for (int i = 0; i < 10; i++)
            {
                if (dispatchMode == "Com")
                {
                    if (port != null)
                    {
                        port.Write(buffer, 0, buffer.Length);
                    }
                }
                else
                {
                    if (UdpConnect != null)
                    {
                        UdpConnect.SendData(buffer, null);
                    }
                }
                Thread.Sleep(sendSplitTime);
            }
            if (writeLogFlag)
            {
                #region 写日志
                string str = "停止点放行AGV（ReleaseAgvOnStop）， 停止点地标： " + mark.ToString() + "\n";
                for (int i = 0; i < buffer.Length; i++)
                    str += Convert.ToString(buffer[i], 16).ToString() + " ";
                gs_blackBoxFile.WriteFile(100, str);
                #endregion
            }
        }
        /// <summary>
        /// 数据解析
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MatchedParity(object sender, DataEventArgs e)
        {
            byte[] buffer;
            if (e.type == 1)
            {
                buffer = e.data;
                buffer.CopyTo(send_buffer, 0);
                if (buffer[COM_FUNC] == 0x33)// && buffer[COM_ROUTE] > 0x30
                {
                    if (port.serialPort1.IsOpen)
                    {
                        //回复AGV
                        send_buffer[COM_FUNC] = 0x34;
                        send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                        port.Write(send_buffer, 0, send_buffer.Length);
                        //通知订阅了AGV到结束点事件处理器
                        AgvDataEventArgs e1 = new AgvDataEventArgs();
                        e1.agvID = buffer[COM_AGV];
                        e1.address = buffer[COM_ADDRESS] - 0x30;
                        /*2016-12-27 薛德龙修改*/
                        e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                        /*2016-12-27 薛德龙修改*/
                        e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                        e1.type = e.type;
                        OnAgvReadEnding(this, e1);
                        if (writeLogFlag)
                        {
                            #region 写日志
                            string str = "";
                            for (int i = 0; i < buffer.Length; i++)
                                str += Convert.ToString(buffer[i], 16).ToString() + " ";
                            gs_blackBoxFile.WriteFile(100, str);
                            #endregion
                        }
                    }
                }
                else if (buffer[COM_FUNC] == 0x32)
                {
                    gs_receive0x32Flag = false;
                    //通知订阅了AGV放行成功事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleased(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x35)
                {
                    send_buffer[COM_FUNC] = 0x36;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了AGV到结束点事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReadStoping(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x38)
                {
                    //通知订阅了AGV放行成功事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                   e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleasedOnStop(this, e1);

                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x41)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x43;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了产线叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                   e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCalled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x42)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x44;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了产线取消叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                   e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCancled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x46)
                {
                    //通知订阅了产线回复45事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                   e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x64)
                {
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();

                    #region 

                    e1.AgvId = (int)buffer[2];
                    e1.AgvMark = Convert.ToInt32(buffer[4] + ((buffer[3] & 0x1F)) * 256);
                    e1.AGVRoute = Convert.ToInt32(buffer[9] + ((buffer[8] & 0x1F)) * 256);

                    e1.ControlCommand = (int)((buffer[3] & 0xE0) >> 5);


                    if (e1.ControlCommand == 1)
                    {

                        switch ((((buffer[8] & 0xE0) >> 5) + 0x40))
                        {
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }

                        //报警信息处理
                        switch (buffer[6] + 1)
                        {
                            #region 异常分类
                            case 1:
                                //agvinfo.ALARMCODE = "1";
                                e1.AgvAlarmStatus = "无异常";
                                break;
                            case 2:
                                e1.AgvAlarmStatus = "急停报警";
                                break;
                            case 3:
                                e1.AgvAlarmStatus = "防撞报警";
                                break;
                            case 4:
                                e1.AgvAlarmStatus = "牵引棒报警";
                                break;
                            case 5:
                                e1.AgvAlarmStatus = "障碍物报警";
                                break;
                            case 6:
                                e1.AgvAlarmStatus = "路线异常";
                                break;
                            case 7:
                                e1.AgvAlarmStatus = "电压异常";
                                break;
                            case 8:
                                e1.AgvAlarmStatus = "脱轨报警";
                                break;
                            case 9:
                                e1.AgvAlarmStatus = "低电压报警";
                                break;
                            case 10:
                                e1.AgvAlarmStatus = "低电压停机";
                                break;
                            case 11:
                                e1.AgvAlarmStatus = "超载报警";
                                break;
                            default:
                                e1.AgvAlarmStatus = "未知异常";
                                break;
                                #endregion
                        }
                        if (buffer[6] != 0)
                        {
                            e1.AgvStatus = buffer[6].ToString();
                        }
                        //速度等级
                        int SpeedGrade = (int)buffer[7];
                        if (SpeedGrade > 0 && SpeedGrade < 6)
                        {
                            e1.SpeedGrade = SpeedGrade.ToString();
                        }

                    }
                    else if (e1.ControlCommand == 2)
                    {
                        int BatteryEleInt = (int)(buffer[5] << 8 | buffer[6]);
                        int BatteryPowInt = (int)buffer[7];
                        e1.Current = ((decimal)BatteryEleInt) / 100;
                        e1.Battery_Power = (decimal)BatteryPowInt;
                    }
                    else if (e1.ControlCommand == 3)
                    {
                        int BatteryVolInt = (int)((buffer[5] << 4) | (buffer[6] >> 4));

                        e1.Voltage = ((decimal)BatteryVolInt) / 100;
                        int BatteryTemInt = (int)(((buffer[6] & 0x0F) << 8) | (buffer[7]));
                        e1.Temperatrue = ((decimal)BatteryTemInt) / 100;
                    }


                    #endregion
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 12).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x62)//
                {
                   
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();
                    int bMarkNum = 0;
                    bMarkNum = buffer[3] | ((buffer[7] & 0xC0) << 2);
                    e1.AgvId = buffer[2];
                    if (bMarkNum>0)
                    {
                       e1.AgvMark = bMarkNum;     
                    }
                    e1.Type = e.type;
                    e1.AGVRoute=buffer[6];
                    {
                        switch ((byte)(buffer[5] & 0x7f))
                        {
                            case 0x01: 
                                e1.AgvStatus="急停" ;
                                break;
                            case 0x02:
                                e1.AgvStatus = "防撞机构err";
                                break;
                            case 0x03:
                                e1.AgvStatus = "牵引棒";
                                break;
                            case 0x04:
                                e1.AgvStatus = "障碍物";
                                break;
                            case 0x05:
                                e1.AgvStatus = "驱动报警";
                                break;
                            case 0x06:
                                e1.AgvStatus = "后驱动异常";
                                break;
                            case 0x07:
                                e1.AgvStatus = "脱轨";
                                break;
                            case 0x08:
                                e1.AgvStatus = "低电压报警";
                                break;
                            case 0x09:
                                e1.AgvStatus = "低电压停机";
                                break;
                            case 0x0A:
                                e1.AgvStatus = "超载";
                                break;
                            case 0x0B:
                                e1.AgvStatus = "驱动提升";
                                break;
                            case 0x0C:
                                e1.AgvStatus = "后牵引";
                                break;
                            case 0x0D:
                                e1.AgvStatus = "";
                                break;
                            case 0x0E:
                                e1.AgvStatus = "启动";
                                break;
                            case 0x0F:
                                e1.AgvStatus = "复位";
                                break;
                            case 0x10:
                                e1.AgvStatus = "方向切换";
                                break;
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }
                    }
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
            }
            else if (e.type == 2)
            {
                buffer = e.data;
                buffer.CopyTo(send_buffer, 0);
                if (buffer[NET_FUNC] == 0x41)
                {
                    //回复AGV
                    send_buffer[NET_FUNC] = 0x43;
                    send_buffer[NET_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    for (int i = 0; i < 3; i++)
                    {
                        CSNetComm.SendBytes(0, send_buffer, send_buffer.Length);
                        Thread.Sleep(200);
                    }
                    //port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了产线叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[NET_AGV];
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[NET_ROUTE - 1] << 8) & 0xFF) + buffer[NET_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    //  e1.route = buffer[NET_ROUTE];
                    e1.mark = ((int)buffer[NET_MARK] << 8 | buffer[NET_MARK + 1]);
                    e1.type = e.type;
                    e1.callStation = buffer[NET_MARK + 1];
                    e1.materialID = buffer[NET_MAID];
                    e1.maNum = buffer[NET_MANUM];
                    e1.PADID = buffer[NET_PADID];
                    e1.callID = buffer[NET_CALLID];
                    OnWorkLineCalled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
               
                else if (buffer[NET_FUNC] == 0x62)//罗国2017-7-20
                {
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();
                    int bMarkNum = 0;
                    bMarkNum = buffer[3] | ((buffer[7] & 0xC0) << 2);
                    e1.AgvId = buffer[2];
                    if (bMarkNum > 0)
                    {
                        e1.AgvMark = bMarkNum;
                    }
                    e1.Type = e.type;
                    e1.AGVRoute = buffer[6];
                    {
                        switch ((byte)(buffer[5] & 0x7f))
                        {
                            case 0x01:
                                e1.AgvStatus = "急停";
                                break;
                            case 0x02:
                                e1.AgvStatus = "防撞机构err";
                                break;
                            case 0x03:
                                e1.AgvStatus = "牵引棒";
                                break;
                            case 0x04:
                                e1.AgvStatus = "障碍物";
                                break;
                            case 0x05:
                                e1.AgvStatus = "驱动报警";
                                break;
                            case 0x06:
                                e1.AgvStatus = "后驱动异常";
                                break;
                            case 0x07:
                                e1.AgvStatus = "脱轨";
                                break;
                            case 0x08:
                                e1.AgvStatus = "低电压报警";
                                break;
                            case 0x09:
                                e1.AgvStatus = "低电压停机";
                                break;
                            case 0x0A:
                                e1.AgvStatus = "超载";
                                break;
                            case 0x0B:
                                e1.AgvStatus = "驱动提升";
                                break;
                            case 0x0C:
                                e1.AgvStatus = "后牵引";
                                break;
                            case 0x0D:
                                e1.AgvStatus = "";
                                break;
                            case 0x0E:
                                e1.AgvStatus = "启动";
                                break;
                            case 0x0F:
                                e1.AgvStatus = "复位";
                                break;
                            case 0x10:
                                e1.AgvStatus = "方向切换";
                                break;
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }
                    }
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[NET_FUNC] == 0x42)
                {
                    //回复AGV
                    send_buffer[NET_FUNC] = 0x44;
                    send_buffer[NET_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    for (int i = 0; i < 3; i++)
                    {
                        CSNetComm.SendBytes(0, send_buffer, send_buffer.Length);
                        Thread.Sleep(200);
                    }

                    //通知订阅了产线取消叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[NET_AGV];
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[NET_ROUTE - 1] << 8) & 0xFF) + buffer[NET_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    //  e1.route = buffer[NET_ROUTE];
                    e1.mark = ((int)buffer[NET_MARK] << 8 | buffer[NET_MARK + 1]);
                    e1.type = e.type;
                    e1.callStation = buffer[NET_MARK + 1];
                    e1.materialID = buffer[NET_MAID];
                    e1.maNum = buffer[NET_MANUM];
                    e1.PADID = buffer[NET_PADID];
                    e1.callID = buffer[NET_CALLID];
                    OnWorkLineCancled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[NET_FUNC] == 0x46)
                {
                    //通知订阅了产线回复45事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[NET_AGV];
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[NET_ROUTE - 1] << 8) & 0xFF) + buffer[NET_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    //  e1.route = buffer[NET_ROUTE];
                    e1.mark = ((int)buffer[NET_MARK] << 8 | buffer[NET_MARK + 1]);
                    e1.type = e.type;
                    e1.callStation = buffer[NET_MARK + 1];
                    e1.materialID = buffer[NET_MAID];
                    e1.maNum = buffer[NET_MANUM];
                    e1.PADID = buffer[NET_PADID];
                    e1.callID = buffer[NET_CALLID];
                    OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
            }
            else if (e.type == 3)
            {
                buffer = e.data;
                buffer.CopyTo(send_buffer, 0);
                if (buffer[COM_FUNC] == 0x33)
                {
                    if (UdpConnect != null)
                    {
                        //回复AGV
                        send_buffer[COM_FUNC] = 0x34;
                        send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                        UdpConnect.SendData(send_buffer, null);

                        //通知订阅了AGV到结束点事件处理器
                        AgvDataEventArgs e1 = new AgvDataEventArgs();
                        e1.agvID = buffer[COM_AGV];
                        e1.address = buffer[COM_ADDRESS] - 0x30;
                        /*2016-12-27 薛德龙修改*/
                        e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                        /*2016-12-27 薛德龙修改*/
                        e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                        e1.type = e.type;
                        OnAgvReadEnding(this, e1);
                        if (writeLogFlag)
                        {
                            #region 写日志
                            string str = "";
                            for (int i = 0; i < buffer.Length; i++)
                                str += Convert.ToString(buffer[i], 16).ToString() + " ";
                            gs_blackBoxFile.WriteFile(100, str);
                            #endregion
                        }
                    }
                }
                else if (buffer[COM_FUNC] == 0x32)
                {
                    gs_receive0x32Flag = false;
                    //通知订阅了AGV放行成功事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleased(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x35)
                {
                    UdpConnect.SendData(send_buffer, null);
                    send_buffer[COM_FUNC] = 0x36;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    UdpConnect.SendData(send_buffer, null);

                    //通知订阅了AGV到结束点事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReadStoping(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x37)
                {
                    UdpConnect.SendData(send_buffer, null);
                    send_buffer[COM_FUNC] = 0x36;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    UdpConnect.SendData(send_buffer, null);

                    //通知订阅了AGV到结束点事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleased37(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x38)
                {
                    //通知订阅了AGV放行成功事件处理器
                    UdpConnect.SendData(send_buffer, null);
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleasedOnStop(this, e1);

                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x41)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x43;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    UdpConnect.SendData(send_buffer, null);

                    //通知订阅了产线叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCalled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x42)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x44;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    UdpConnect.SendData(send_buffer, null);

                    //通知订阅了产线取消叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCancled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x46)
                {
                    //通知订阅了产线回复45事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x81)//V4盒子回复的协议
                {
                    V4DataEventArgs e1 = new V4DataEventArgs();

                    e1.num = buffer[2];
                    #region
                    e1.Y16 = ((int)(buffer[3] & 128) == 128 ? 1 : 0);
                    e1.Y15 = ((int)(buffer[3] & 64) == 64 ? 1 : 0);
                    e1.Y14 = ((int)(buffer[3] & 32) == 32 ? 1 : 0);
                    e1.Y13 = ((int)(buffer[3] & 16) == 16 ? 1 : 0);
                    e1.Y12 = ((int)(buffer[3] & 8) == 8 ? 1 : 0);
                    e1.Y11 = ((int)(buffer[3] & 4) == 4 ? 1 : 0);
                    e1.Y10 = ((int)(buffer[3] & 2) == 2 ? 1 : 0);
                    e1.Y09 = ((int)(buffer[3] & 1) == 1 ? 1 : 0);

                    e1.Y08 = ((int)(buffer[4] & 128) == 128 ? 1 : 0);
                    e1.Y07 = ((int)(buffer[4] & 64) == 64 ? 1 : 0);
                    e1.Y06 = ((int)(buffer[4] & 32) == 32 ? 1 : 0);
                    e1.Y05 = ((int)(buffer[4] & 16) == 16 ? 1 : 0);
                    e1.Y04 = ((int)(buffer[4] & 8) == 8 ? 1 : 0);
                    e1.Y03 = ((int)(buffer[4] & 4) == 4 ? 1 : 0);
                    e1.Y02 = ((int)(buffer[4] & 2) == 2 ? 1 : 0);
                    e1.Y01 = ((int)(buffer[4] & 1) == 1 ? 1 : 0);

                    e1.X16 = ((int)(buffer[5] & 128) == 128 ? 1 : 0);
                    e1.X15 = ((int)(buffer[5] & 64) == 64 ? 1 : 0);
                    e1.X14 = ((int)(buffer[5] & 32) == 32 ? 1 : 0);
                    e1.X13 = ((int)(buffer[5] & 16) == 16 ? 1 : 0);
                    e1.X12 = ((int)(buffer[5] & 8) == 8 ? 1 : 0);
                    e1.X11 = ((int)(buffer[5] & 4) == 4 ? 1 : 0);
                    e1.X10 = ((int)(buffer[5] & 2) == 2 ? 1 : 0);
                    e1.X09 = ((int)(buffer[5] & 1) == 1 ? 1 : 0);

                    e1.X08 = ((int)(buffer[6] & 128) == 128 ? 1 : 0);
                    e1.X07 = ((int)(buffer[6] & 64) == 64 ? 1 : 0);
                    e1.X06 = ((int)(buffer[6] & 32) == 32 ? 1 : 0);
                    e1.X05 = ((int)(buffer[6] & 16) == 16 ? 1 : 0);
                    e1.X04 = ((int)(buffer[6] & 8) == 8 ? 1 : 0);
                    e1.X03 = ((int)(buffer[6] & 4) == 4 ? 1 : 0);
                    e1.X02 = ((int)(buffer[6] & 2) == 2 ? 1 : 0);
                    e1.X01 = ((int)(buffer[6] & 1) == 1 ? 1 : 0);
                    #endregion
                    OnV4Data(e1);
                }
                else if (buffer[COM_FUNC]==0x64)
                {
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();

                    #region 
                    
                    e1.AgvId = (int)buffer[2];
                    e1.AgvMark = Convert.ToInt32(buffer[4] + ((buffer[3] & 0x1F)) * 256);
                    e1.AGVRoute= Convert.ToInt32(buffer[9] + ((buffer[8] & 0x1F)) * 256);

                    e1.ControlCommand = (int)((buffer[3] & 0xE0) >> 5);


                    if (e1.ControlCommand == 1)
                    {

                        switch ((((buffer[8] & 0xE0) >> 5) + 0x40))
                        {
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }

                        //报警信息处理
                        switch (buffer[6] + 1)
                        {
                            #region 异常分类
                            case 1:
                                //agvinfo.ALARMCODE = "1";
                                e1.AgvAlarmStatus = "无异常";
                                break;
                            case 2:
                                e1.AgvAlarmStatus = "急停报警";
                                break;
                            case 3:
                                e1.AgvAlarmStatus = "防撞报警";
                                break;
                            case 4:
                                e1.AgvAlarmStatus = "牵引棒报警";
                                break;
                            case 5:
                                e1.AgvAlarmStatus = "障碍物报警";
                                break;
                            case 6:
                                e1.AgvAlarmStatus = "路线异常";
                                break;
                            case 7:
                                e1.AgvAlarmStatus = "电压异常";
                                break;
                            case 8:
                                e1.AgvAlarmStatus = "脱轨报警";
                                break;
                            case 9:
                                e1.AgvAlarmStatus = "低电压报警";
                                break;
                            case 10:
                                e1.AgvAlarmStatus = "低电压停机";
                                break;
                            case 11:
                                e1.AgvAlarmStatus = "超载报警";
                                break;
                            default:
                                e1.AgvAlarmStatus = "未知异常";
                                break;
                                #endregion
                        }
                        if (buffer[6] != 0)
                        {
                            e1.AgvStatus = buffer[6].ToString(); 
                        }
                        //速度等级
                        int SpeedGrade = (int)buffer[7];
                        if (SpeedGrade > 0 && SpeedGrade < 6)
                        {
                            e1.SpeedGrade = SpeedGrade.ToString();
                        }
                      
                    }
                    else if (e1.ControlCommand == 2)
                    {
                        int BatteryEleInt = (int)(buffer[5] << 8 | buffer[6]);
                        int BatteryPowInt = (int)buffer[7];
                        e1.Current = ((decimal)BatteryEleInt) / 100;
                        e1.Battery_Power = (decimal)BatteryPowInt;
                    }
                    else if (e1.ControlCommand == 3)
                    {
                        int BatteryVolInt = (int)((buffer[5] << 4) | (buffer[6] >> 4));

                        e1.Voltage = ((decimal)BatteryVolInt) / 100;
                        int BatteryTemInt = (int)(((buffer[6] & 0x0F) << 8) | (buffer[7]));
                        e1.Temperatrue = ((decimal)BatteryTemInt) / 100;
                    }


                    #endregion
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 12).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x62)//
                {
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();
                    int bMarkNum = 0;
                    bMarkNum = buffer[3] | ((buffer[7] & 0xC0) << 2);
                    e1.AgvId = buffer[2];
                    if (bMarkNum > 0)
                    {
                        e1.AgvMark = bMarkNum;
                    }
                    e1.Type = e.type;
                    e1.AGVRoute = buffer[6];
                    {
                        switch ((byte)(buffer[5] & 0x7f))
                        {
                            case 0x01:
                                e1.AgvStatus = "急停";
                                break;
                            case 0x02:
                                e1.AgvStatus = "防撞机构err";
                                break;
                            case 0x03:
                                e1.AgvStatus = "牵引棒";
                                break;
                            case 0x04:
                                e1.AgvStatus = "障碍物";
                                break;
                            case 0x05:
                                e1.AgvStatus = "驱动报警";
                                break;
                            case 0x06:
                                e1.AgvStatus = "后驱动异常";
                                break;
                            case 0x07:
                                e1.AgvStatus = "脱轨";
                                break;
                            case 0x08:
                                e1.AgvStatus = "低电压报警";
                                break;
                            case 0x09:
                                e1.AgvStatus = "低电压停机";
                                break;
                            case 0x0A:
                                e1.AgvStatus = "超载";
                                break;
                            case 0x0B:
                                e1.AgvStatus = "驱动提升";
                                break;
                            case 0x0C:
                                e1.AgvStatus = "后牵引";
                                break;
                            case 0x0D:
                                e1.AgvStatus = "";
                                break;
                            case 0x0E:
                                e1.AgvStatus = "启动";
                                break;
                            case 0x0F:
                                e1.AgvStatus = "复位";
                                break;
                            case 0x10:
                                e1.AgvStatus = "方向切换";
                                break;
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }
                    }
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 10).ToString() + " ";
                        gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
            }
        }//end of MatchedParity
        #endregion
    }
}

